/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import static java.util.Objects.*;

import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.Antragsteller;
import de.ozgcloud.eingang.common.formdata.FormData;

@Component
class AfmAntragstellerMapper implements AfmEngineBasedMapper {

	static final String POSTFACH_ID = "nameid";

	static final String ANTRAGSTELLER = "antragsteller";
	static final String ANTRAGSTELLER_UPPERCASE = "Antragsteller";
	static final String ANREDE = "b_anrede";
	static final String VORNAME = "pers_vorname";
	static final String NACHNAME = "pers_nachname";
	static final String GEBURTSNAME = "pers_geburtsname";
	static final String GEBURTSDATUM = "pers_geburtsdatum";
	static final String GEBURTSORT = "pers_geburtsort";
	static final String EMAIL = "kont_email";
	static final String TELEFON = "kont_telefonnummer";
	static final String STRASSE = "sh_strasse";
	static final String HAUSNUMMER = "sh_hausnummer";
	static final String PLZ = "sh_plz";
	static final String ORT = "ort";

	static final String FIELD_FIRMA_NAME = "firma_name";
	static final String FIELD_FIRMA_STRASSE = "firma_strasse";
	static final String FIELD_FIRMA_HAUSNUMMER = "firma_hausnummer";
	static final String FIELD_FIRMA_PLZ = "firma_plz";
	static final String FIELD_FIRMA_STADT = "firma_stadt";
	static final String FIELD_ANSPRECHPARTNER_ANREDE = "firma_ansprechpartner_anrede";
	static final String FIELD_ANSPRECHPARTNER_VORNAME = "firma_ansprechpartner_vorname";
	static final String FIELD_ANSPRECHPARTNER_NACHNAME = "firma_ansprechpartner_nachname";

	@Autowired
	private AfmAntragstellerHeaderMapper antragstellerHeaderMapper;

	@Override
	public FormData parseFormData(FormData formData) {
		if (antragstellerHeaderMapper.isResponsible(formData)) {
			return antragstellerHeaderMapper.parseAntragstellerData(formData);
		}
		var formDataMap = formData.getFormData();
		var builder = Antragsteller.builder().postfachId(getPostfachId(formDataMap));

		var antragsteller = getAntragstellerMap(formDataMap)
				.map(antragstellerMap -> addAntragstellerData(builder, antragstellerMap))
				.orElse(builder)
				.build();

		var cleanedMap = removeMappedData(formDataMap);
		return formData.toBuilder().antragsteller(antragsteller).formData(cleanedMap).build();
	}

	private String getPostfachId(Map<String, Object> formDataMap) {
		return (String) formDataMap.get(POSTFACH_ID);
	}

	private Optional<Map<String, Object>> getAntragstellerMap(Map<String, Object> formDataMap) {
		return Optional.ofNullable(formDataMap.get(ANTRAGSTELLER))
				.or(() -> Optional.ofNullable(formDataMap.get(ANTRAGSTELLER_UPPERCASE)))
				.map(Map.class::cast)
				.map(LinkedHashMap<String, Object>::new);
	}

	private Antragsteller.AntragstellerBuilder addAntragstellerData(Antragsteller.AntragstellerBuilder builder,
			Map<String, Object> antragstellerMap) {
		return isNull(antragstellerMap.get(FIELD_FIRMA_NAME))
				? addPrivateAntragstellerData(builder, antragstellerMap)
				: addFirmAntragstellerData(builder, antragstellerMap);
	}

	private Antragsteller.AntragstellerBuilder addPrivateAntragstellerData(Antragsteller.AntragstellerBuilder builder,
			Map<String, Object> antragstellerMap) {
		return builder //
				.anrede((String) antragstellerMap.get(ANREDE)) //
				.vorname((String) antragstellerMap.get(VORNAME)) //
				.nachname((String) antragstellerMap.get(NACHNAME)) //
				.geburtsname((String) antragstellerMap.get(GEBURTSNAME)) //
				.geburtsdatum((String) antragstellerMap.get(GEBURTSDATUM)) //
				.geburtsort((String) antragstellerMap.get(GEBURTSORT)) //
				.email((String) antragstellerMap.get(EMAIL)) //
				.telefon((String) antragstellerMap.get(TELEFON)) //
				.strasse((String) antragstellerMap.get(STRASSE)) //
				.hausnummer((String) antragstellerMap.get(HAUSNUMMER)) //
				.plz((String) antragstellerMap.get(PLZ)) //
				.ort((String) antragstellerMap.get(ORT)) //
				.data(getNotMappedData(antragstellerMap, getPrivateFields()));
	}

	Antragsteller.AntragstellerBuilder addFirmAntragstellerData(Antragsteller.AntragstellerBuilder builder,
			Map<String, Object> antragstellerMap) {
		return builder
				.firmaName(MapUtils.getString(antragstellerMap, FIELD_FIRMA_NAME))
				.strasse(MapUtils.getString(antragstellerMap, FIELD_FIRMA_STRASSE))
				.hausnummer(MapUtils.getString(antragstellerMap, FIELD_FIRMA_HAUSNUMMER))
				.plz(MapUtils.getString(antragstellerMap, FIELD_FIRMA_PLZ))
				.ort(MapUtils.getString(antragstellerMap, FIELD_FIRMA_STADT))
				.anrede(MapUtils.getString(antragstellerMap, FIELD_ANSPRECHPARTNER_ANREDE))
				.vorname(MapUtils.getString(antragstellerMap, FIELD_ANSPRECHPARTNER_VORNAME))
				.nachname(MapUtils.getString(antragstellerMap, FIELD_ANSPRECHPARTNER_NACHNAME))
				.telefon(MapUtils.getString(antragstellerMap, TELEFON))
				.email(MapUtils.getString(antragstellerMap, EMAIL))
				.data(getNotMappedData(antragstellerMap, getFirmaFields()));
	}

	private Map<String, Object> getNotMappedData(Map<String, Object> antragstellerMap, Set<String> mappedFields) {
		return antragstellerMap.entrySet().stream().filter(entry -> !mappedFields.contains(entry.getKey()))
				.collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue, (key1, key2) -> key1, LinkedHashMap::new));
	}

	private Set<String> getPrivateFields() {
		return Set.of(ANREDE, VORNAME, NACHNAME, GEBURTSNAME, GEBURTSDATUM, GEBURTSORT, EMAIL, TELEFON, STRASSE, HAUSNUMMER, PLZ, ORT);
	}

	private Set<String> getFirmaFields() {
		return Set.of(FIELD_FIRMA_NAME, FIELD_FIRMA_STRASSE, FIELD_FIRMA_HAUSNUMMER, FIELD_FIRMA_PLZ, FIELD_FIRMA_STADT,
				FIELD_ANSPRECHPARTNER_ANREDE, FIELD_ANSPRECHPARTNER_VORNAME, FIELD_ANSPRECHPARTNER_NACHNAME, TELEFON, EMAIL);
	}

	private Map<String, Object> removeMappedData(Map<String, Object> formDataMap) {
		var editableMap = new LinkedHashMap<>(formDataMap);
		if (nonNull(editableMap.get(ANTRAGSTELLER))) {
			editableMap.remove(ANTRAGSTELLER);
		} else {
			editableMap.remove(ANTRAGSTELLER_UPPERCASE);
		}
		return Collections.unmodifiableMap(editableMap);
	}
}
